!function(e,n){"function"==typeof define&&define.amd?require(["jquery"],n):n(e.jQuery)}("undefined"!=typeof self?self:this,function(e){"use strict";function n(){e.map(e(".sl-youtube-video"),o)}function i(e){var n=e.target.getIframe();t[n.id]=t[n.id]||n.height/n.width,n.width="100%",o(n),n.style.visibility="visible"}function o(e){e.height=e.offsetWidth*t[e.id]}e(window).on("resize",function(e){return function(){window.requestAnimationFrame(e)}}(n));var t={};e(function(){var n={playVideo:function(e,i){n.loadPlayer(e,i)},loadPlayer:function(n,o){var t={videoId:null,playerVars:{autoplay:0,controls:1,rel:0,showInfo:0,origin:window.location.origin},events:{onReady:i}},a=e.extend(t,o);void 0!==a["playerVars-start"]&&(a.playerVars.start=a["playerVars-start"]),new window.YT.Player(n,a)}},o=function(){var i=e(".sl-youtube-video");i.length&&(window.onYouTubePlayerAPIReady=function(){i.each(function(i,o){var t=e(o);n.playVideo(t.attr("id"),t.data("youtube"))})},"undefined"==typeof YT||void 0===window.YT.Player?e.getScript("//www.youtube.com/iframe_api"):window.onYouTubePlayerAPIReady())};e(document).on("onBeforeClose",".overlay",o),o()})}),define("videoblock",function(){}),require(["videoblock"],function(e){}),define("contenttype",function(){});