import Toolbox from "toolbox/Toolbox";
import Simplelayout from "simplelayout/Simplelayout"
import Cropper from "cropper/Cropper"
import 'dropzone';

module.exports = {
  simplelayout: Simplelayout,
  toolbox: Toolbox,
  Cropper: Cropper
}
