@if $standard-iconset == font-awesome {

  @each $type, $value in get-portal-type-icons-for-iconset(font-awesome) {
    body .sl-toolbox a.sl-toolbox-block .sl-icon-#{$type} {
      @extend body.icons-on .contenttype-#{$type};
      &:before {
        color: inherit;
      }
    }
  }

}
